@@ -5,16 +5,16 @@
 # ruff: noqa: W291
 # ruff: noqa: D400
 # ruff: noqa: E501
-
 import json
 import logging
-from typing import Literal
+from typing import Any, Literal
 
 import requests
 
 from .grvt_ccxt_base import GrvtCcxtBase
 from .grvt_ccxt_env import GrvtEnv, get_grvt_endpoint
 from .grvt_ccxt_types import (
+    Amount,
     GrvtInstrumentKind,
     GrvtInvalidOrder,
     GrvtOrderSide,
@@ -51,15 +51,16 @@ class GrvtCcxt(GrvtCcxtBase):
         env: GrvtEnv,
         logger: logging.Logger | None = None,
         parameters: dict = {},
+        order_book_ccxt_format: bool = False,
     ):
         """Initialize the GrvtCcxt instance."""
-        super().__init__(env, logger, parameters)
+        super().__init__(env, logger, parameters, order_book_ccxt_format)
         self._clsname: str = type(self).__name__
         self._session: requests.Session = requests.Session()
         self._session.headers.update({"Content-Type": "application/json"})
         self.refresh_cookie()
         # Assign markets here
-        self.markets = self.load_markets()
+        self.markets: dict[str, dict] = self.load_markets()
 
     def refresh_cookie(self) -> dict | None:
         """Refresh the session cookie."""
@@ -75,8 +76,7 @@ class GrvtCcxt(GrvtCcxtBase):
                     {"X-Grvt-Account-Id": self._cookie["X-Grvt-Account-Id"]}
                 )
             self.logger.info(
-                f"refresh_cookie {self._cookie=} {self._session.cookies=}"
-                f" {self._session.headers=}"
+                f"refresh_cookie {self._cookie=} {self._session.cookies=} {self._session.headers=}"
             )
         return self._cookie
 
@@ -93,15 +93,14 @@ class GrvtCcxt(GrvtCcxtBase):
         payload_json = json.dumps(payload, cls=EnumEncoder)
         self.logger.info(f"{FN} {payload=}\n{payload_json=}")
         return_value = self._session.post(path, data=payload_json, timeout=5)
+        return_text: str = ""
         try:
             return_text = return_value.text
             response = return_value.json()
         except Exception as err:
             self.logger.warning(f"{FN} Unable to parse {return_value=} as json. {err=}")
         if not return_value.ok:
-            self.logger.warning(
-                f"{FN} ERROR {payload_json=}\n{return_value=}\n{response=}"
-            )
+            self.logger.warning(f"{FN} ERROR {payload_json=}\n{return_value=}\n{response=}")
         else:
             if len(return_text) > MAX_LEN_TO_LOG:
                 self.logger.debug(f"{FN} OK {return_value=} {response=}")
@@ -134,14 +133,14 @@ class GrvtCcxt(GrvtCcxtBase):
             f"{FN} Order created:"
             f"{response.get('result', {}).get('metadata', {}).get('client_order_id')}"
         )
-        return response.get("result")
+        return response.get("result", {})
 
     def create_order(
         self,
         symbol: str,
         order_type: GrvtOrderType,
         side: GrvtOrderSide,
-        amount: Num,
+        amount: Amount,
         price: Num = None,
         params={},
     ) -> dict:
@@ -153,7 +152,7 @@ class GrvtCcxt(GrvtCcxtBase):
         # create GrvtOrder object
         order_duration_secs = params.get("order_duration_secs", 24 * 60 * 60)
         order = get_grvt_order(
-            sub_account_id=self._trading_account_id,
+            sub_account_id=self.get_trading_account_id(),
             symbol=symbol,
             order_type=order_type,
             side=side,
@@ -168,7 +167,7 @@ class GrvtCcxt(GrvtCcxtBase):
         self,
         symbol: str,
         side: GrvtOrderSide,
-        amount: Num,
+        amount: Amount,
         price: Num = None,
         params={},
     ) -> dict:
@@ -204,7 +203,7 @@ class GrvtCcxt(GrvtCcxtBase):
         self,
         id: str | None = None,
         symbol: str | None = None,
-        params: dict | None = {},
+        params: dict = {},
     ) -> bool:
         """
         Ccxt compliant signature
@@ -216,7 +215,7 @@ class GrvtCcxt(GrvtCcxtBase):
         Args:
             id (str): exchange assigned order ID<br>
             symbol (str): trading symbol<br>
-            params: 
+            params:
                 * client_order_id (str): client assigned order ID<br>
                 * time_to_live_ms (str): lifetime of cancel requiest in millisecs<br>
         Returns:
@@ -225,7 +224,7 @@ class GrvtCcxt(GrvtCcxtBase):
         FN = f"{self._clsname} cancel_order"
         self._check_account_auth()
         payload: dict = {
-            "sub_account_id": str(self._trading_account_id),
+            "sub_account_id": self.get_trading_account_id(),
         }
         if id:
             payload["order_id"] = str(id)
@@ -239,7 +238,7 @@ class GrvtCcxt(GrvtCcxtBase):
 
         path = get_grvt_endpoint(self.env, "CANCEL_ORDER")
         self.logger.info(
-            f"{FN} Send cancel {payload=} for trading_account_id={self._trading_account_id}"
+            f"{FN} Send cancel {payload=} for trading_account_id={self.get_trading_account_id()}"
         )
         response: dict = self._auth_and_post(path, payload)
         cancel_ack = response.get("result", {}).get("ack")
@@ -285,16 +284,13 @@ class GrvtCcxt(GrvtCcxtBase):
         open_orders: list = response.get("result", [])
         if symbol:
             open_orders = [
-                o
-                for o in open_orders
-                if o.get("legs") and o["legs"][0].get("instrument") == symbol
+                o for o in open_orders if o.get("legs") and o["legs"][0].get("instrument") == symbol
             ]
         return open_orders
 
     def fetch_order(
         self,
         id: str | None = None,
-        symbol: str = None,
         params: dict = {},
     ) -> dict:
         """
@@ -305,14 +301,13 @@ class GrvtCcxt(GrvtCcxtBase):
         for details.<br>
         Args:
             id: (str) order_id to fetch.<br>
-            symbol: (str) get orders for this symbol only.<br>
             params: dictionary with parameters. Valid keys:<br>
                 `client_order_id` (int): client assigned order ID.<br>
         Return: dict with order's details or {} if order was NOT found.
         """
         self._check_account_auth()
         payload = {
-            "sub_account_id": str(self._trading_account_id),
+            "sub_account_id": self.get_trading_account_id(),
         }
         if id:
             payload["order_id"] = id
@@ -320,8 +315,7 @@ class GrvtCcxt(GrvtCcxtBase):
             payload["client_order_id"] = str(params["client_order_id"])
         else:
             raise GrvtInvalidOrder(
-                f"{self._clsname} fetch_order() requires order_id "
-                "or params['client_order_id']"
+                f"{self._clsname} fetch_order() requires order_id or params['client_order_id']"
             )
         path = get_grvt_endpoint(self.env, "GET_ORDER")
         response: dict = self._auth_and_post(path, payload)
@@ -355,12 +349,12 @@ class GrvtCcxt(GrvtCcxtBase):
         return response
 
     def get_account_summary(
-        self, type: Literal["sub-account", "funding", "aggregated"]
+        self, type: Literal["sub-account", "funding", "aggregated"] = "sub-account"
     ) -> dict:
         """
         Return: The account summary.
         Private call requires authorization.<br>
-        See [Account Summary](https://api-docs.grvt.io/trading_api/#account_summary)
+        See [Account Summary](https://api-docs.grvt.io/trading_api/#sub-account_summary)
         for details.<br>
         Returns: dictionary with account data.<br>.
         """
@@ -369,7 +363,7 @@ class GrvtCcxt(GrvtCcxtBase):
         payload = {}
         if type == "sub-account":
             path = get_grvt_endpoint(self.env, "GET_ACCOUNT_SUMMARY")
-            payload = {"sub_account_id": str(self._trading_account_id)}
+            payload = {"sub_account_id": self.get_trading_account_id()}
         elif type == "funding":
             path = get_grvt_endpoint(self.env, "GET_FUNDING_ACCOUNT_SUMMARY")
         elif type == "aggregated":
@@ -383,7 +377,25 @@ class GrvtCcxt(GrvtCcxtBase):
             self.logger.info(f"{FN} No account summary for {path=} {payload=}")
         return sub_account
 
-    def fetch_account_history(self, params: dict = {}) -> dict:
+    def fetch_balance(self, type: Literal["sub-account", "funding", "aggregated"] = "sub-account") -> dict:
+        """
+        Ccxt compliant signature
+        Fetch balances for the account.<br>
+        Private call requires authorization.<br>
+        See [Account Summary](https://api-docs.grvt.io/trading_api/#sub-account_summary)
+            for details.<br>.
+
+        Args:
+            type: (str) - The type of account to fetch balances for. Defaults to 'sub-account'.
+                Valid values: 'sub-account', 'funding', 'aggregated'.
+
+        Returns: dictionary with ccxt-compliant balance data https://docs.ccxt.com/#/README?id=account-balance.<br>.
+        """
+        account_summary: dict = self.get_account_summary(type)
+        return self._get_balances_from_account_summary(account_summary)
+
+
+    def fetch_account_history(self, params: dict = {}, limit: int = 500) -> dict:
         """
         HISTORICAL data.<br>
         Get account history.<br>
@@ -392,6 +404,7 @@ class GrvtCcxt(GrvtCcxtBase):
             for details.<br>.
 
         Args:
+            limit: maximum number of account snapshots per page to fetch.<br>
             params: dictionary with parameters. Valid keys:<br>
                 `start_time` (int): fetch orders since this timestamp in nanoseconds.<br>
                 `end_time` (int): fetch orders until this timestamp in nanoseconds.<br>
@@ -405,7 +418,7 @@ class GrvtCcxt(GrvtCcxtBase):
         """
         self._check_account_auth()
         # Prepare request payload
-        payload = self._get_payload_fetch_account_history(params)
+        payload = self._get_payload_fetch_account_history(limit, params)
         # Post payload and parse the response
         path = get_grvt_endpoint(self.env, "GET_ACCOUNT_HISTORY")
         response: dict = self._auth_and_post(path, payload=payload)
@@ -482,7 +495,7 @@ class GrvtCcxt(GrvtCcxtBase):
         return response
 
     # **************** PUBLIC API CALLS
-    def load_markets(self) -> dict:
+    def load_markets(self) -> dict[str, dict]:
         self.logger.info("load_markets START")
         instruments = self.fetch_markets(
             params={
@@ -493,7 +506,7 @@ class GrvtCcxt(GrvtCcxtBase):
         )
         if instruments:
             self.markets = {
-                i.get("instrument"): i for i in instruments if i.get("instrument")
+                str(i.get("instrument", "")): i for i in instruments if i.get("instrument")
             }
             self.logger.info(f"load_markets: loaded {len(self.markets)} markets.")
         else:
@@ -613,6 +626,9 @@ class GrvtCcxt(GrvtCcxtBase):
             payload["depth"] = limit
         path = get_grvt_endpoint(self.env, "GET_ORDER_BOOK")
         response: dict = self._auth_and_post(path, payload=payload)
+        if self.is_order_book_ccxt_format():
+            # Convert to ccxt format
+            return self.convert_grvt_ob_to_ccxt(response.get("result", {}))
         return response.get("result", {})
 
     def fetch_recent_trades(
@@ -629,7 +645,7 @@ class GrvtCcxt(GrvtCcxtBase):
         # 'is_taker_buyer': True, 'size': '24000000000', 'price': '2600000000000',
         # 'mark_price': '2591055564869', 'index_price': '2592459142472', 'interest_rate': 0,
         # 'forward_price': '0', 'trade_id': '729726', 'venue': 'ORDERBOOK'}
-        payload = {"instrument": symbol}
+        payload: dict[str, str | int] = {"instrument": symbol}
         if limit:
             payload["limit"] = limit
         path = get_grvt_endpoint(self.env, "GET_TRADES")
@@ -642,24 +658,24 @@ class GrvtCcxt(GrvtCcxtBase):
         since: int | None = None,
         limit: int = 10,
         params: dict = {},
-    ) -> list:
+    ) -> dict:
         """
         Ccxt compliant signature, HISTORICAL data.<br>
         Retrieve trade history of a given instrument.
         :param symbol: The instrument name.
-        :return: The list of trades.
+        :return: dict with field 'result' containing a list of trades.
         """
         #  List of {'event_time': '1724248876870635916', 'instrument': 'ETH_USDT_Perp',
         # 'is_taker_buyer': True, 'size': '24000000000', 'price': '2600000000000',
         # 'mark_price': '2591055564869', 'index_price': '2592459142472', 'interest_rate': 0,
         # 'forward_price': '0', 'trade_id': '729726', 'venue': 'ORDERBOOK'}
-        payload = self._get_payload_fetch_trades(
+        payload: dict = self._get_payload_fetch_trades(
             symbol,
             since=since,
             limit=limit,
             params=params,
         )
-        path = get_grvt_endpoint(self.env, "GET_TRADE_HISTORY")
+        path: str = get_grvt_endpoint(self.env, "GET_TRADE_HISTORY")
         response: dict = self._auth_and_post(path, payload=payload)
         return response
 
@@ -667,38 +683,38 @@ class GrvtCcxt(GrvtCcxtBase):
         self,
         symbol: str,
         since: int = 0,
-        limit: int = 10,
+        limit: int = 1_000,
         params: dict = {},
-    ) -> list:
+    ) -> dict:
         """
         Ccxt compliant signature, HISTORICAL data.<br>
         Retrieve the funding rates history of a given instrument.<br>
         Args:
-            symbol: The instrument name.<br>
-            since: fetch trades since this timestamp in nanoseconds.<br>
-            limit: maximum number of trades to fetch.<br>
+            symbol (str): The instrument name.<br>
+            since (int): fetch trades since this timestamp in nanoseconds.<br>
+            limit: int - maximum number of trades to fetch.<br>
             params: dictionary with parameters. Valid keys:<br>
                 `cursor` (str): cursor for the pagination.
                             If cursor is present then we ignore other filters.<br>
                 `end_time` (int): end time in nanoseconds.<br>
         Returns:
-            list of dictionaries repesenting funding rate at a point in time with fields:<br>
+            dict with field 'result' containing list of dictionaries repesenting funding rate
+            at a point in time with fields:<br>
                 `instrument` (str): instrument name.<br>
                 'funding_rate' (float): funding rate.<br>
                 'funding_time' (int): funding time in nanoseconds.<br>
                 'mark_price' (float): mark price.<br>.
         """
-        payload = {"instrument": symbol}
+        payload: dict[str, str | int] = {"instrument": symbol}
         if params.get("cursor"):
             payload["cursor"] = params["cursor"]
         else:
             if since:
-                payload["start_time"] = since
+                payload["start_time"] = str(since)
             if params.get("end_time"):
-                payload["end_time"] = params["end_time"]
+                payload["end_time"] = str(params["end_time"])
             if limit:
-                payload["limit"] = limit
-
+                payload["limit"] = int(limit)
         path = get_grvt_endpoint(self.env, "GET_FUNDING")
         response: dict = self._auth_and_post(path, payload=payload)
         return response
@@ -710,7 +726,7 @@ class GrvtCcxt(GrvtCcxtBase):
         since: int = 0,
         limit: int = 10,
         params={},
-    ) -> list:
+    ) -> dict:
         """
         Ccxt compliant signature, HISTORICAL data.
 
@@ -727,13 +743,7 @@ class GrvtCcxt(GrvtCcxtBase):
                 `end_time` (int): end time in nanoseconds.
                 `candle_type` (str): candle type. Valid values: 'TRADE', 'MARK', 'INDEX'.
         Returns:
-            list of dictionaries repesenting funding rate at a point in time with fields:<br>
-                `instrument` (str): instrument name.<br>
-                'funding_rate' (float): funding rate.<br>
-                'funding_time' (int): funding time in nanoseconds.<br>
-                'mark_price' (float): mark price.<br>
-        Returns:
-            a list of dictionaries, each dict representing a candlestick with fields:<br>
+            dict with field 'result' containing a list of dictionaries, each dict representing a candlestick with fields:<br>
                 `instrument` - instrument name.<br>
                 `open_time` - start of interval in nanoseconds.<br>
                 `close_time` - end of interval in nanoseconds.<br>
@@ -746,7 +756,9 @@ class GrvtCcxt(GrvtCcxtBase):
                 `trades` - number of trades.<br>
         """
         FN = f"{self._clsname} fetch_ohlcv"
-        payload = self._get_payload_fetch_ohlcv(symbol, timeframe, since, limit, params)
+        payload: dict[str, Any] = self._get_payload_fetch_ohlcv(
+            symbol, timeframe, since, limit, params
+        )
         self.logger.info(f"{FN} {payload=}")
         path = get_grvt_endpoint(self.env, "GET_CANDLESTICK")
         response: dict = self._auth_and_post(path, payload=payload)
