@@ -18,6 +18,7 @@ from .grvt_ccxt_base import GrvtCcxtBase
 # from env import ENDPOINTS
 from .grvt_ccxt_env import GrvtEnv, get_grvt_endpoint
 from .grvt_ccxt_types import (
+    Amount,
     GrvtInstrumentKind,
     GrvtInvalidOrder,
     GrvtOrderSide,
@@ -54,13 +55,12 @@ class GrvtCcxtPro(GrvtCcxtBase):
         env: GrvtEnv,
         logger: logging.Logger | None = None,
         parameters: dict = {},
+        order_book_ccxt_format: bool = False,
     ):
         """Initialize the GrvtCcxt instance."""
-        super().__init__(env, logger, parameters)
+        super().__init__(env, logger, parameters, order_book_ccxt_format)
         self._clsname: str = type(self).__name__
-        self._session = aiohttp.ClientSession(
-            headers={"Content-Type": "application/json"}
-        )
+        self._session = aiohttp.ClientSession(headers={"Content-Type": "application/json"})
         # Force sync call to get cookie here
         self._cookie = get_cookie_with_expiration(
             get_grvt_endpoint(self.env, "AUTH"), self._api_key
@@ -83,18 +83,13 @@ class GrvtCcxtPro(GrvtCcxtBase):
         """Refresh the session cookie."""
         if not self.should_refresh_cookie():
             return self._cookie
-        path = get_grvt_endpoint(self.env, "AUTH")
+        path: str = get_grvt_endpoint(self.env, "AUTH")
         self._cookie = await get_cookie_with_expiration_async(path, self._api_key)
         self._path_return_value_map[path] = self._cookie
         self.update_session_with_cookie()
         return self._cookie
 
     # PRIVATE API CALLS
-    async def _check_valid_symbol_async(self, symbol: str) -> None:
-        # if not self.markets:
-        #     self.markets = await self.load_markets()
-        return super()._check_valid_symbol(symbol)
-
     async def _auth_and_post(self, path: str, payload: dict) -> dict:
         FN = f"{self._clsname} _auth_and_post {path=}"
         MAX_LEN_TO_LOG = 1280
@@ -106,12 +101,14 @@ class GrvtCcxtPro(GrvtCcxtBase):
         await self.refresh_cookie()
         payload_json = json.dumps(payload, cls=EnumEncoder)
         self.logger.info(f"{FN} {payload=}\n{payload_json=}")
+        return_text: str = ""
         async with self._session.post(
             url=path,
             data=payload_json,
             headers={"Content-Type": "application/json"},
             timeout=5,
         ) as return_value:
+            return_text: str = ""
             try:
                 return_text = await return_value.text()
                 response = await return_value.json(content_type="application/json")
@@ -154,16 +151,16 @@ class GrvtCcxtPro(GrvtCcxtBase):
             f"{FN} Order created:"
             f"{response.get('result', {}).get('metadata', {}).get('client_order_id')}"
         )
-        return response.get("result")
+        return response.get("result", {})
 
     def _get_order_with_validations(
         self,
         symbol: str,
         order_type: GrvtOrderType,
         side: GrvtOrderSide,
-        amount: Num,
+        amount: Amount,
         price: Num = None,
-        params={},
+        params: dict = {},
     ) -> GrvtOrder:
         self._check_account_auth()
         self._check_valid_symbol(symbol)
@@ -172,7 +169,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
         # create GrvtOrder object
         order_duration_secs = params.get("order_duration_secs", 24 * 60 * 60)
         return get_grvt_order(
-            sub_account_id=self._trading_account_id,
+            sub_account_id=self.get_trading_account_id(),
             symbol=symbol,
             order_type=order_type,
             side=side,
@@ -187,21 +184,19 @@ class GrvtCcxtPro(GrvtCcxtBase):
         symbol: str,
         order_type: GrvtOrderType,
         side: GrvtOrderSide,
-        amount: Num,
+        amount: Amount,
         price: Num = None,
         params={},
     ) -> dict:
         """Ccxt compliant signature."""
-        order = self._get_order_with_validations(
-            symbol, order_type, side, amount, price, params
-        )
+        order = self._get_order_with_validations(symbol, order_type, side, amount, price, params)
         return await self._create_grvt_order(order)
 
     async def create_limit_order(
         self,
         symbol: str,
         side: GrvtOrderSide,
-        amount: Num,
+        amount: Amount,
         price: Num = None,
         params={},
     ) -> dict:
@@ -249,7 +244,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
         Args:
             id (str): exchange assigned order ID<br>
             symbol (str): trading symbol<br>
-            params: 
+            params:
                 * client_order_id (str): client assigned order ID<br>
                 * time_to_live_ms (str): lifetime of cancel requiest in millisecs<br>
         Returns:
@@ -319,16 +314,13 @@ class GrvtCcxtPro(GrvtCcxtBase):
         open_orders: list = response.get("result", [])
         if symbol:
             open_orders = [
-                o
-                for o in open_orders
-                if o.get("legs") and o["legs"][0].get("instrument") == symbol
+                o for o in open_orders if o.get("legs") and o["legs"][0].get("instrument") == symbol
             ]
         return open_orders
 
     async def fetch_order(
         self,
         id: str | None = None,
-        symbol: str | None = None,
         params: dict = {},
     ) -> dict:
         """
@@ -356,9 +348,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
         elif "client_order_id" in params:
             payload["client_order_id"] = str(params["client_order_id"])
         else:
-            raise GrvtInvalidOrder(
-                f"{FN} requires either order_id or params['client_order_id']"
-            )
+            raise GrvtInvalidOrder(f"{FN} requires either order_id or params['client_order_id']")
         path = get_grvt_endpoint(self.env, "GET_ORDER")
         response: dict = await self._auth_and_post(path, payload)
         return response
@@ -391,7 +381,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
         return response
 
     async def get_account_summary(
-        self, type: Literal["sub-account", "funding", "aggregated"]
+        self, type: Literal["sub-account", "funding", "aggregated"] = "sub-account"
     ) -> dict:
         """
         Return: The account summary.
@@ -419,7 +409,26 @@ class GrvtCcxtPro(GrvtCcxtBase):
             self.logger.info(f"{FN} No account summary for {path=} {payload=}")
         return sub_account
 
-    async def fetch_account_history(self, params: dict = {}) -> dict:
+    async def fetch_balance(
+        self, type: Literal["sub-account", "funding", "aggregated"] = "sub-account"
+    ) -> dict:
+        """
+        Ccxt compliant signature
+        Fetch balances for the account.<br>
+        Private call requires authorization.<br>
+        See [Account Summary](https://api-docs.grvt.io/trading_api/#sub-account_summary)
+            for details.<br>.
+
+        Args:
+            type: (str) - The type of account to fetch balances for. Defaults to 'sub-account'.
+                Valid values: 'sub-account', 'funding', 'aggregated'.
+
+        Returns: dictionary with ccxt-compliant balance data https://docs.ccxt.com/#/README?id=account-balance.<br>.
+        """
+        account_summary: dict = await self.get_account_summary(type)
+        return self._get_balances_from_account_summary(account_summary)
+
+    async def fetch_account_history(self, params: dict = {}, limit: int = 500) -> dict:
         """
         HISTORICAL data.<br>
         Get account history.<br>
@@ -428,6 +437,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
             for details.<br>.
 
         Args:
+            limit: maximum number of account snapshots per page to fetch.<br>
             params: dictionary with parameters. Valid keys:<br>
                 `start_time` (int): fetch orders since this timestamp in nanoseconds.<br>
                 `end_time` (int): fetch orders until this timestamp in nanoseconds.<br>
@@ -441,7 +451,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
         """
         self._check_account_auth()
         # Prepare request payload
-        payload = self._get_payload_fetch_account_history(params)
+        payload = self._get_payload_fetch_account_history(limit, params)
         # Post payload and parse the response
         path = get_grvt_endpoint(self.env, "GET_ACCOUNT_HISTORY")
         response: dict = await self._auth_and_post(path, payload=payload)
@@ -518,7 +528,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
         return response
 
     # **************** PUBLIC API CALLS
-    async def load_markets(self) -> dict:
+    async def load_markets(self) -> dict | None:
         self.logger.info("load_markets START")
         instruments = await self.fetch_markets(
             params={
@@ -646,6 +656,9 @@ class GrvtCcxtPro(GrvtCcxtBase):
             payload["depth"] = limit
         path = get_grvt_endpoint(self.env, "GET_ORDER_BOOK")
         response: dict = await self._auth_and_post(path, payload=payload)
+        if self.is_order_book_ccxt_format():
+            # Convert to ccxt format
+            return self.convert_grvt_ob_to_ccxt(response.get("result", {}))
         return response.get("result", {})
 
     async def fetch_recent_trades(
@@ -662,7 +675,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
         # 'is_taker_buyer': True, 'size': '24000000000', 'price': '2600000000000',
         # 'mark_price': '2591055564869', 'index_price': '2592459142472', 'interest_rate': 0,
         # 'forward_price': '0', 'trade_id': '729726', 'venue': 'ORDERBOOK'}
-        payload = {"instrument": symbol}
+        payload: dict[str, str | int] = {"instrument": symbol}
         if limit:
             payload["limit"] = limit
         path = get_grvt_endpoint(self.env, "GET_TRADES")
@@ -675,24 +688,24 @@ class GrvtCcxtPro(GrvtCcxtBase):
         since: int | None = None,
         limit: int = 10,
         params: dict = {},
-    ) -> list:
+    ) -> dict:
         """
-        ccxt-compliant signature, HISTORICAL data.<br>
+        Ccxt-compliant signature, HISTORICAL data.<br>
         Retrieve trade history of a given instrument.
         :param symbol: The instrument name.
-        :return: The list of trades.
+        :return: dict with field 'result' containing a list of trades.
         """
         #  List of {'event_time': '1724248876870635916', 'instrument': 'ETH_USDT_Perp',
         # 'is_taker_buyer': True, 'size': '24000000000', 'price': '2600000000000',
         # 'mark_price': '2591055564869', 'index_price': '2592459142472', 'interest_rate': 0,
         # 'forward_price': '0', 'trade_id': '729726', 'venue': 'ORDERBOOK'}
-        payload = self._get_payload_fetch_trades(
+        payload: dict = self._get_payload_fetch_trades(
             symbol,
             since=since,
             limit=limit,
             params=params,
         )
-        path = get_grvt_endpoint(self.env, "GET_TRADE_HISTORY")
+        path: str = get_grvt_endpoint(self.env, "GET_TRADE_HISTORY")
         response: dict = await self._auth_and_post(path, payload=payload)
         return response
 
@@ -700,38 +713,39 @@ class GrvtCcxtPro(GrvtCcxtBase):
         self,
         symbol: str,
         since: int = 0,
-        limit: int = 10,
+        limit: int = 1_000,
         params: dict = {},
-    ) -> list:
+    ) -> dict:
         """
         ccxt-compliant signature, HISTORICAL data.<br>
         Retrieve the funding rates history of a given instrument.<br>
         Args:
-            symbol: The instrument name.<br>
-            since: fetch trades since this timestamp in nanoseconds.<br>
-            limit: maximum number of trades to fetch.<br>
+            symbol (str): The instrument name.<br>
+            since (int): fetch trades since this timestamp in nanoseconds.<br>
+            limit: int - maximum number of trades to fetch.<br>
             params: dictionary with parameters. Valid keys:<br>
                 `cursor` (str): cursor for the pagination.
                             If cursor is present then we ignore other filters.<br>
                 `end_time` (int): end time in nanoseconds.<br>
         Returns:
-            list of dictionaries repesenting funding rate at a point in time with fields:<br>
+            dict with field 'result' containing list of dictionaries repesenting funding rate
+            at a point in time with fields:<br>
                 `instrument` (str): instrument name.<br>
                 'funding_rate' (float): funding rate.<br>
                 'funding_time' (int): funding time in nanoseconds.<br>
                 'mark_price' (float): mark price.<br>.
         """
-        payload = {"instrument": symbol}
+        payload: dict[str, str | int] = {"instrument": symbol}
         if params.get("cursor"):
             payload["cursor"] = params["cursor"]
         else:
             if since:
-                payload["start_time"] = since
+                payload["start_time"] = str(since)
             if params.get("end_time"):
-                payload["end_time"] = params["end_time"]
+                payload["end_time"] = str(params["end_time"])
             if limit:
-                payload["limit"] = limit
-        path = get_grvt_endpoint(self.env, "GET_FUNDING")
+                payload["limit"] = int(limit)
+        path: str = get_grvt_endpoint(self.env, "GET_FUNDING")
         response: dict = await self._auth_and_post(path, payload=payload)
         return response
 
@@ -742,7 +756,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
         since: int = 0,
         limit: int = 10,
         params={},
-    ) -> list:
+    ) -> dict:
         """
         ccxt-compliant signature, HISTORICAL data.<br>
         Retrieve the ohlc history of a given instrument.<br>
@@ -758,13 +772,7 @@ class GrvtCcxtPro(GrvtCcxtBase):
                 `end_time` (int): end time in nanoseconds.<br>
                 `candle_type` (str): candle type. Valid values: 'TRADE', 'MARK', 'INDEX'.<br>
         Returns:
-            list of dictionaries repesenting funding rate at a point in time with fields:<br>
-                `instrument` (str): instrument name.<br>
-                'funding_rate' (float): funding rate.<br>
-                'funding_time' (int): funding time in nanoseconds.<br>
-                'mark_price' (float): mark price.<br>
-        Returns:
-            a list of dictionaries, each dict representing a candlestick with fields:<br>
+            dict with field 'result' containing a list of dictionaries, each dict representing a candlestick with fields:<br>
                 `instrument` - instrument name.<br>
                 `open_time` - start of interval in nanoseconds.<br>
                 `close_time` - end of interval in nanoseconds.<br>
@@ -776,9 +784,9 @@ class GrvtCcxtPro(GrvtCcxtBase):
                 `volume_q` - volume in quote(USDT).<br>
                 `trades` - number of trades.<br>.
         """
-        FN = f"{self._clsname} fetch_ohlcv"
-        payload = self._get_payload_fetch_ohlcv(symbol, timeframe, since, limit, params)
+        FN: str = f"{self._clsname} fetch_ohlcv"
+        payload: dict = self._get_payload_fetch_ohlcv(symbol, timeframe, since, limit, params)
         self.logger.info(f"{FN} {payload=}")
-        path = get_grvt_endpoint(self.env, "GET_CANDLESTICK")
+        path: str = get_grvt_endpoint(self.env, "GET_CANDLESTICK")
         response: dict = await self._auth_and_post(path, payload=payload)
         return response
